"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const sequelize_1 = require("sequelize");
const sql_comparison_builder_1 = require("./sql-comparison.builder");
/**
 * @internal
 * Builds a WHERE clause from a Filter.
 */
class WhereBuilder {
    constructor(sqlComparisionBuilder = new sql_comparison_builder_1.SQLComparisionBuilder()) {
        this.sqlComparisionBuilder = sqlComparisionBuilder;
    }
    /**
     * Builds a WHERE clause from a Filter.
     * @param filter - the filter to build the WHERE clause from.
     */
    build(filter) {
        const { and, or } = filter;
        let ands = [];
        let ors = [];
        let whereOpts = {};
        if (and && and.length) {
            ands = and.map((f) => this.build(f));
        }
        if (or && or.length) {
            ors = or.map((f) => this.build(f));
        }
        const filterAnds = this.filterFields(filter);
        if (filterAnds) {
            ands = [...ands, filterAnds];
        }
        if (ands.length) {
            whereOpts = { ...whereOpts, [sequelize_1.Op.and]: ands };
        }
        if (ors.length) {
            whereOpts = { ...whereOpts, [sequelize_1.Op.or]: ors };
        }
        return whereOpts;
    }
    /**
     * Creates field comparisons from a filter. This method will ignore and/or properties.
     * @param filter - the filter with fields to create comparisons for.
     */
    filterFields(filter) {
        const ands = Object.keys(filter)
            .filter((f) => f !== 'and' && f !== 'or')
            .map((field) => this.withFilterComparison(field, this.getField(filter, field)));
        if (ands.length === 1) {
            return ands[0];
        }
        if (ands.length) {
            return { [sequelize_1.Op.and]: ands };
        }
        return undefined;
    }
    getField(obj, field) {
        return obj[field];
    }
    withFilterComparison(field, cmp) {
        const opts = Object.keys(cmp);
        if (opts.length === 1) {
            const cmpType = opts[0];
            return this.sqlComparisionBuilder.build(field, cmpType, cmp[cmpType]);
        }
        return {
            [sequelize_1.Op.or]: opts.map((cmpType) => this.sqlComparisionBuilder.build(field, cmpType, cmp[cmpType])),
        };
    }
}
exports.WhereBuilder = WhereBuilder;
//# sourceMappingURL=where.builder.js.map